unit GenUtils;

interface

uses
  Windows, Messages, Registry, Controls;
  
function IntIndex(I: Integer; const A: array of Integer): Integer;
function HighSetBit(I: Cardinal): Integer;

type
  TRange = record
    Min, Max: Extended;
  end;

function EnsureRange(const AValue: Extended; const ARange: TRange): Extended; overload;

const
  _2Pi = 2 * Pi;

function dBToLin(const dB: Extended): Extended;
function LinTodB(const Lin: Extended): Extended;
function MiddBLevel(const L: array of Extended): Extended;

function RoundToLen(const AValue: Double; ADigits: Integer): Double;
function FracLen(const AValue: Extended; ADigits: Integer): Integer;

resourcestring
  SDays = 'd';

function TimeToStrEx(const Time: TDateTime): string;

procedure SetEnableControls(const AControls: array of TControl; AEnable: Boolean);
procedure PlaceControl(AControl: TControl; const APoint: TPoint);
procedure ChangeWindowStyle(Handle: HWND; Incl, Excl: DWORD);


const
  SWindowLeft       = 'WindowLeft';
  SWindowTop        = 'WindowTop';
  
var
  CompanyName, ProductName: string;

function RegKey: string;


implementation

uses SysUtils, Math;


function IntIndex(I: Integer; const A: array of Integer): Integer;
begin
  for Result := 0 to High(A) do
    if A[Result] = I then
      Exit;
  Result := -1;
end;

function HighSetBit(I: Cardinal): Integer;
asm
        BSR     EAX,EAX
end;


function EnsureRange(const AValue: Extended; const ARange: TRange): Extended;
begin
  Result := AValue;
  with ARange do
  begin
    Assert(Min <= Max);
    if Result < Min then
      Result := Min;
    if Result > Max then
      Result := Max;
  end;
end;


function dBToLin(const dB: Extended): Extended;
begin
  Result := Power(2, dB / 6);
end;

function LinTodB(const Lin: Extended): Extended;
begin
  Result := 6 * Log2(Lin);
end;

function MiddBLevel(const L: array of Extended): Extended;
var
  Sum: Extended;
  I: Integer;
begin
  Sum := 0;
  for I := 0 to High(L) do
    Sum := Sum + dBToLin(L[I]);
  Result := LinTodB(Sum / Length(L));
end;


function RoundToLen(const AValue: Double; ADigits: Integer): Double;
begin
  Result := RoundTo(AValue, -FracLen(AValue, ADigits));
end;

function FracLen(const AValue: Extended; ADigits: Integer): Integer;
begin
  if AValue <> 0.0 then
    Result := ADigits - Trunc(Log10(Abs(AValue))) - 1
  else
    Result := 0;
end;


function TimeToStrEx(const Time: TDateTime): string;
begin
  DateTimeToString(Result, 'HH:mm:ss', Time);
  if Time >= 1 then
    Result := Format('%d%s %s', [Trunc(Time), SDays, Result]);
end;


procedure SetEnableControls(const AControls: array of TControl; AEnable: Boolean);
var
  i: Integer;
begin
  for i := 0 to High(AControls) do
    AControls[i].Enabled := AEnable;
end;

procedure PlaceControl(AControl: TControl; const APoint: TPoint);
begin
  with AControl, APoint do
    SetBounds(X - Width div 2, Y - Height div 2, Width, Height);
end;

procedure ChangeWindowStyle(Handle: HWND; Incl, Excl: DWORD);
var
  AStyle: DWORD;
begin
  AStyle := GetWindowLong(Handle, GWL_STYLE);
  if AStyle <> 0 then
    SetWindowLong(Handle, GWL_STYLE, (AStyle or Incl) and not Excl);
end;


function RegKey: string;
begin
  Result := '\Software\' + CompanyName + '\' + ProductName;
end;

initialization
  CompanyName := 'MyCompany';
  ProductName := 'MyProduct';

end.
